@@ -12,7 +12,10 @@ use xxhash_rust::xxh3::Xxh3;
 
 use crate::{
     Packet,
-    packet::Iter,
+    packet::{
+        Iter,
+        SEIP,
+    },
 };
 
 /// A packet's body holds either unprocessed bytes, processed bytes,
@@ -71,8 +74,8 @@ pub enum Body {
     ///     that we failed to process, say because we didn't support
     ///     the packet's version.
     ///
-    ///   - The packet is an encryption container ([`SEIP`] or
-    ///     [`AED`]) and the body is encrypted.
+    ///   - The packet is an encryption container ([`SEIP`]) and the
+    ///     body is encrypted.
     ///
     /// Note: if some of a packet's data is streamed, and the
     /// `PacketParser` is configured to buffer unread content, then
@@ -82,7 +85,6 @@ pub enum Body {
     ///   [`Literal`]: crate::packet::Literal
     ///   [`Unknown`]: crate::packet::Unknown
     ///   [`SEIP`]: crate::packet::SEIP
-    ///   [`AED`]: crate::packet::AED
     Unprocessed(Vec<u8>),
 
     /// Processed packed body.
@@ -131,13 +133,6 @@ pub struct Container {
 
 assert_send_and_sync!(Container);
 
-impl std::ops::Deref for Container {
-    type Target = Body;
-    fn deref(&self) -> &Self::Target {
-        &self.body
-    }
-}
-
 impl PartialEq for Container {
     fn eq(&self, other: &Container) -> bool {
         use Body::*;
@@ -297,13 +292,9 @@ impl Container {
 
     /// Returns the hash for the empty body.
     fn empty_body_digest() -> u64 {
-        lazy_static::lazy_static!{
-            static ref DIGEST: u64 = {
-                Container::make_body_hash().digest()
-            };
-        }
-
-        *DIGEST
+        use std::sync::OnceLock;
+        static DIGEST: OnceLock<u64> = OnceLock::new();
+        *DIGEST.get_or_init(|| Container::make_body_hash().digest())
     }
 
     /// Creates a hash context for hashing the body.
@@ -323,7 +314,8 @@ impl Container {
         format!("{:08X}", self.body_digest)
     }
 
-    // Converts an indentation level to whitespace.
+    /// Converts an indentation level to whitespace.
+    #[cfg(test)]
     fn indent(depth: usize) -> &'static str {
         use std::cmp;
 
@@ -331,16 +323,17 @@ impl Container {
         &s[0..cmp::min(depth, s.len())]
     }
 
-    // Pretty prints the container to stderr.
-    //
-    // This function is primarily intended for debugging purposes.
-    //
-    // `indent` is the number of spaces to indent the output.
+    /// Pretty prints the container to stderr.
+    ///
+    /// This function is primarily intended for debugging purposes.
+    ///
+    /// `indent` is the number of spaces to indent the output.
+    #[cfg(test)]
     pub(crate) fn pretty_print(&self, indent: usize) {
         for (i, p) in self.children_ref().iter().enumerate() {
             eprintln!("{}{}: {:?}",
                       Self::indent(indent), i + 1, p);
-            if let Some(ref children) = self.children_ref()
+            if let Some(children) = self.children_ref()
                 .and_then(|c| c.get(i)).and_then(|p| p.container_ref())
             {
                 children.pretty_print(indent + 1);
@@ -349,7 +342,7 @@ impl Container {
     }
 }
 
-macro_rules! impl_body_forwards {
+macro_rules! impl_unprocessed_body_forwards {
     ($typ:ident) => {
         /// This packet implements the unprocessed container
         /// interface.
@@ -394,14 +387,45 @@ macro_rules! impl_body_forwards {
     };
 }
 
+macro_rules! impl_processed_body_forwards {
+    ($typ:ident) => {
+        /// This packet implements the processed container
+        /// interface.
+        ///
+        /// Container packets like this one can contain either
+        /// unprocessed or processed, structured data.
+        impl $typ {
+            /// Returns a reference to the container.
+            pub fn container_ref(&self) -> &packet::Container {
+                &self.container
+            }
+
+            /// Returns a mutable reference to the container.
+            pub fn container_mut(&mut self) -> &mut packet::Container {
+                &mut self.container
+            }
+
+            /// Gets a reference to the this packet's body.
+            pub fn body(&self) -> &crate::packet::Body {
+                self.container_ref().body()
+            }
+
+            /// Sets the this packet's body.
+            pub fn set_body(&mut self, body: crate::packet::Body)
+                            -> crate::packet::Body {
+                self.container_mut().set_body(body)
+            }
+        }
+    };
+}
+
 impl Packet {
     pub(crate) // for packet_pile.rs
     fn container_ref(&self) -> Option<&Container> {
-        use std::ops::Deref;
         match self {
-            Packet::CompressedData(p) => Some(p.deref()),
-            Packet::SEIP(p) => Some(p.deref()),
-            Packet::AED(p) => Some(p.deref()),
+            Packet::CompressedData(p) => Some(p.container_ref()),
+            Packet::SEIP(SEIP::V1(p)) => Some(p.container_ref()),
+            Packet::SEIP(SEIP::V2(p)) => Some(p.container_ref()),
             Packet::Literal(p) => Some(p.container_ref()),
             Packet::Unknown(p) => Some(p.container_ref()),
             _ => None,
@@ -410,11 +434,10 @@ impl Packet {
 
     pub(crate) // for packet_pile.rs, packet_pile_parser.rs, parse.rs
     fn container_mut(&mut self) -> Option<&mut Container> {
-        use std::ops::DerefMut;
         match self {
-            Packet::CompressedData(p) => Some(p.deref_mut()),
-            Packet::SEIP(p) => Some(p.deref_mut()),
-            Packet::AED(p) => Some(p.deref_mut()),
+            Packet::CompressedData(p) => Some(p.container_mut()),
+            Packet::SEIP(SEIP::V1(p)) => Some(p.container_mut()),
+            Packet::SEIP(SEIP::V2(p)) => Some(p.container_mut()),
             Packet::Literal(p) => Some(p.container_mut()),
             Packet::Unknown(p) => Some(p.container_mut()),
             _ => None,
@@ -427,7 +450,7 @@ impl Packet {
         self.container_ref().and_then(|c| c.children())
     }
 
-    /// Returns an iterator over all of the packet's descendants, in
+    /// Returns an iterator over all the packet's descendants, in
     /// depth-first order.
     pub(crate) fn descendants(&self) -> Option<Iter> {
         self.container_ref().and_then(|c| c.descendants())
@@ -435,6 +458,7 @@ impl Packet {
 
     /// Retrieves the packet's unprocessed body.
     #[cfg(test)]
+    #[allow(dead_code)] // Not used if no compression feature is enabled.
     pub(crate) fn unprocessed_body(&self) -> Option<&[u8]> {
         self.container_ref().and_then(|c| match c.body() {
             Body::Unprocessed(bytes) => Some(&bytes[..]),
